@@ -1,4 +1,4 @@
-﻿/*
+/*
  *
  *  Copyright (c) 2014-2015
  *  name : Francis Banyikwa
@@ -26,15 +26,22 @@
 #include <QDebug>
 #include <QFile>
 
+#define DEBUG 0
+
 using cs = siritask::status ;
 
 static bool _create_folder( const QString& m )
 {
-	if( utility::pathExists( m ) ){
+	if( utility::platformIsWindows() ){
 
-		return utility::reUseMountPoint() ;
+		return true ;
 	}else{
-		return utility::createFolder( m ) ;
+		if( utility::pathExists( m ) ){
+
+			return utility::reUseMountPoint() ;
+		}else{
+			return utility::createFolder( m ) ;
+		}
 	}
 }
 
@@ -115,84 +122,116 @@ static QString _wrap_su( const QString& s )
 	}
 }
 
+static std::pair< bool,utility::Task > _unmount_volume( const QString& exe,
+							const QString& mountPoint,
+							bool usePolkit )
+{
+	auto e = utility::preUnMountCommand() ;
+
+	int timeOut = 10000 ;
+
+	if( e.isEmpty() ){
+
+		return { true,utility::Task::run( exe,timeOut,usePolkit ).get() } ;
+	}else{
+		if( utility::Task::run( e + " " + mountPoint,timeOut,false ).get().success() ){
+
+			return { true,utility::Task::run( exe,timeOut,usePolkit ).get() } ;
+		}else{
+			return { false,utility::Task() } ;
+		}
+	}
+}
+
+static bool _unmount_ecryptfs( const QString& cipherFolder,
+			       const QString& mountPoint,
+			       int maxCount )
+{
+	bool not_set = true ;
+
+	auto cmd = [ & ](){
+
+		auto exe = utility::executableFullPath( "ecryptfs-simple" ) ;
+
+		auto s = exe + " -k " + cipherFolder ;
+
+		if( utility::useSiriPolkit() ){
+
+			return _wrap_su( s ) ;
+		}else{
+			return s ;
+		}
+	} ;
+
+	for( int i = 0 ; i < maxCount ; i++ ){
+
+		auto s = _unmount_volume( cmd(),mountPoint,true ) ;
+
+		if( s.first && s.second.success() ){
+
+			return true ;
+		}else{
+			if( not_set && s.second.stdError().contains( "error: failed to set gid" ) ){
+
+				if( utility::enablePolkit( utility::background_thread::True ) ){
+
+					not_set = false ;
+				}else{
+					return false ;
+				}
+			}else{
+				utility::Task::waitForOneSecond() ;
+			}
+		}
+	}
+
+	return false ;
+}
+
+static bool _unmount_rest( const QString& mountPoint,int maxCount )
+{
+	auto cmd = [ & ](){
+
+		if( utility::platformIsOSX() ){
+
+			return "umount " + mountPoint ;
+		}else{
+			return "fusermount -u " + mountPoint ;
+		}
+	}() ;
+
+	for( int i = 0 ; i < maxCount ; i++ ){
+
+		auto s = _unmount_volume( cmd,mountPoint,false ) ;
+
+		if( s.first && s.second.success() ){
+
+			return true ;
+		}else{
+			utility::Task::waitForOneSecond() ;
+		}
+	}
+
+	return false ;
+}
+
 Task::future< bool >& siritask::encryptedFolderUnMount( const QString& cipherFolder,
 							const QString& mountPoint,
 							const QString& fileSystem )
 {
-	return Task::run< bool >( [ = ](){
-
-		auto ecryptfs = _ecryptfs( fileSystem ) ;
-
-		auto cmd = [ & ](){
-
-			if( ecryptfs ){
-
-				auto exe = utility::executableFullPath( "ecryptfs-simple" ) ;
-
-				auto s = exe + " -k " + _makePath( cipherFolder ) ;
-
-				if( utility::useSiriPolkit() ){
-
-					return _wrap_su( s ) ;
-				}else{
-					return s ;
-				}
-			}else{
-				if( utility::platformIsOSX() ){
-
-					return "umount " + _makePath( mountPoint ) ;
-				}else{
-					return "fusermount -u " + _makePath( mountPoint ) ;
-				}
-			}
-		} ;
+	return Task::run( [ = ](){
 
 		const int max_count = 5 ;
 
-		if( ecryptfs ){
+		if( _ecryptfs( fileSystem ) ){
 
-			bool not_set = true ;
+			auto a = _makePath( cipherFolder ) ;
+			auto b = _makePath( mountPoint ) ;
 
-			auto exe = cmd() ;
-
-			for( int i = 0 ; i < max_count ; i++ ){
-
-				auto s = utility::Task::run( exe,10000,true ).get() ;
-
-				if( s.success() ){
-
-					return true ;
-				}else{
-					if( not_set && s.stdError().contains( "error: failed to set gid" ) ){
-
-						if( utility::enablePolkit( utility::background_thread::True ) ){
-
-							not_set = false ;
-
-							exe = cmd() ;
-						}else{
-							return false ;
-						}
-					}else{
-						utility::Task::waitForOneSecond() ;
-					}
-				}
-			}
+			return _unmount_ecryptfs( a,b,max_count ) ;
 		}else{
-			for( int i = 0 ; i < max_count ; i++ ){
-
-				auto s = utility::Task::run( cmd(),10000,false ).get() ;
-
-				if( s.success() ){
-
-					return true ;
-				}else{
-					utility::Task::waitForOneSecond() ;
-				}
-			}
+			return _unmount_rest( _makePath( mountPoint ),max_count ) ;
 		}
-
-		return false ;
 	} ) ;
 }
 
@@ -273,8 +312,8 @@ static QString _args( const QString& exe,const siritask::options& opt,
 
 				if( create ){
 
-					auto e = QString( "%1 --init -q %2 %3" ) ;
-					return e.arg( exe,configPath,cipherFolder ) ;
+					auto e = QString( "%1 --init -q %2 %3 %4" ) ;
+					return e.arg( exe,opt.createOptions,configPath,cipherFolder ) ;
 				}else{
 					mode += " -o fsname=gocryptfs@" + cipherFolder ;
 
@@ -569,8 +608,13 @@ static siritask::cmdStatus _cmd( bool create,const siritask::options& opt,
 
 		return _status( app,status_type::exeNotFound ) ;
 	}else{
+		exe = utility::Task::makePath( exe ) ;
+
 		auto _run = [ & ](){
 
+			#if DEBUG
+				utility::debug() << _args( exe,opt,configFilePath,create ) ;
+			#endif
 			auto s = utility::Task( _args( exe,opt,configFilePath,create ),
 						20000,
 						utility::systemEnvironment(),
@@ -613,7 +657,7 @@ static QString _configFilePath( const siritask::options& opt )
 Task::future< siritask::cmdStatus >& siritask::encryptedFolderMount( const siritask::options& opt,
 								     bool reUseMountPoint )
 {
-	return Task::run< siritask::cmdStatus >( [ opt,reUseMountPoint ]()->siritask::cmdStatus{
+	return Task::run( [ opt,reUseMountPoint ]()->siritask::cmdStatus{
 
 		auto _mount = [ reUseMountPoint ]( const QString& app,const siritask::options& copt,
 				const QString& configFilePath )->siritask::cmdStatus{
@@ -705,7 +749,7 @@ Task::future< siritask::cmdStatus >& siritask::encryptedFolderMount( const sirit
 
 Task::future< siritask::cmdStatus >& siritask::encryptedFolderCreate( const siritask::options& opt )
 {
-	return Task::run< siritask::cmdStatus >( [ opt ]()->siritask::cmdStatus{
+	return Task::run( [ opt ]()->siritask::cmdStatus{
 
 		if( _ecryptfs_illegal_path( opt ) ){
 
